@@ -1,17 +1,10 @@
 #!/usr/bin/perl -w
 #
-# Yarr - Yet Another RSS Reader
+# yarssr - Yet Another RSS Reader
 # Copyright (C) 2004 Lee Aylward <lee@laylward.com>
 # http://yarssr.sourceforge.net
 # Licensed under the GPL
 #
-# TODO: 
-# 	-GConf for settings
-# 	-Audio notification option
-# 	-Menu position to avoid covering panel
-# 	-Separate intervals
-# 	-Splice simplelist directly
-#
 
 use strict;
 use XML::RSS qw( parse );
@@ -19,13 +12,13 @@ use Gtk2::TrayIcon;
 use Gtk2::GladeXML;
 use Gtk2::SimpleList;
 use Gnome2::VFS;
-use Carp;
 use Gnome2;
+use Carp;
 
-croak("this version of Gnome2 is old, you need > 0.94")
+croak("your version the Gnome2 perl module is old, you need > 0.94")
 	if $Gnome2::VERSION < 0.94;
 
-# Shut up annoying XML::RSS warnings. (I am evil apparently :)
+# Shut up annoying XML::RSS warnings.
 BEGIN { $SIG{'__WARN__'} = sub {warn $_[0] unless($_[0]=~/1432\.$/) } }
 $0 = 'yarssr';
 
@@ -33,9 +26,19 @@ use constant TRUE => 1;
 use constant FALSE => 0;
 use constant NEW => 2;
 
+my $name = 'yarssr';
 my $version = '0.1.5';
+my $copyright = 'GNU General Public License (GPL)';
+my $comments = 'http://yarssr.sf.net';
+my $authors = 'Lee Aylward <lee@laylward.com>
+
+Patches from:
+	James Curbo,
+	Dan Lenski';
+
+my $application = Gnome2::Program->init($0, $version);
+
 my %feeds;
-my @feedlist;
 my $parser = new XML::RSS;
 my $menu;
 
@@ -51,7 +54,7 @@ my $config = $ENV{HOME}.'/.yarssr/config';
 
 # Default browser
 my $browser = '/usr/bin/mozilla';
-my $usegnome = FALSE;
+my $usegnome;
 
 # Gtk 2 stuff
 Gtk2->init;
@@ -81,6 +84,13 @@ my $feedlist = Gtk2::SimpleList->new (
 	'Address'	=> 'text',
 );
 
+#$feedlist->set_column_editable(0,TRUE);
+#$feedlist->set_column_editable(2,TRUE);
+
+$feedlist->set_reorderable(TRUE);
+my @columns = $feedlist->get_columns;
+$columns[$_]->set_sort_column_id($_) for (0..2);
+
 # Add simplelist to scrolled window in prefs window
 my $scrolledwindow = $gld->get_widget('scrolledwindow_feeds');
 $scrolledwindow->add($feedlist);
@@ -198,9 +208,78 @@ my @pixmap_new = (
 '                          ',
 '                          ');
 
+my @dot = (
+"16 16 28 1",
+"       c None",
+".      c #A70505",
+"+      c #AA0606",
+"@      c #AE0808",
+"#      c #AF0808",
+"\$      c #B50B0B",
+"%      c #B60B0B",
+"&      c #B90C0C",
+"*      c #BB0E0E",
+"=      c #BC0E0E",
+"-      c #BE1010",
+";      c #BF1010",
+">      c #C01111",
+",      c #C11212",
+"'      c #C31414",
+")      c #C41515",
+"!      c #C51616",
+"~      c #C61818",
+"{      c #C91C1C",
+"]      c #CA1D1D",
+"^      c #CB1E1E",
+"/      c #CF2626",
+"(      c #D12828",
+"_      c #D73535",
+":      c #D83838",
+"<      c #DC4040",
+"[      c #DF4A4A",
+"}      c #E86B6B",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"      '!)>      ",
+"     )(<:~=     ",
+"     )_}[]*     ",
+"     ,^/{,\$     ",
+"     &;-*%+     ",
+"      ##@.      ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ");
+
+my @blank = (
+"16 16 1 1",
+"       c None",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ",
+"                ");
+
 # Make pixbufs out of pixmaps
 my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_xpm_data(@pixmap);
 my $pixbuf_new = Gtk2::Gdk::Pixbuf->new_from_xpm_data(@pixmap_new);
+my $dot = Gtk2::Gdk::Pixbuf->new_from_xpm_data(@dot);
+my $blank = Gtk2::Gdk::Pixbuf->new_from_xpm_data(@blank);
 
 my $image = Gtk2::Image->new_from_pixbuf($pixbuf);
 $eventbox->add($image);
@@ -212,11 +291,6 @@ $tray->show_all;
 read_config();
 update_feed_all();
 
-# Disable Gnome default browser: see NOTES up top
-#$usegnome = FALSE;
-#my $checkbutton = $gld->get_widget('use_default_browser_checkbox');
-#$checkbutton->set_inconsistent(TRUE);
-
 # Create timer with specified interval
 my $timer = Glib::Timeout->add($interval,\&update_feed_all);
 
@@ -253,7 +327,7 @@ sub read_config
 			chomp;
 			if (/^feed=(.*);(.*);(\d)/) 
 			{
-				push @feedlist, [$2,$3,$1];
+				push @{$feedlist->{data}}, [$2,$3,$1];
 				$feeds{$1} = {
 						title => $2,
 						enable=> $3,
@@ -276,7 +350,6 @@ sub read_config
 				$usegnome = $1;
 			}
 		}
-		update_simplelist();
 		close(CONFIG)
 	}
 }
@@ -309,13 +382,14 @@ sub get_feed
 {
 	my ($url,undef) = @_;
 	Gtk2->main_iteration while Gtk2->events_pending; 
-	my ($content,$type) = get_file($url);
+	my ($content,undef) = get_file($url);
 
 	# Check if we got content
 	if ($content)
 	{
 		# Parse content
-		parse_feed($content,$url);
+		# in eval() because XML::RSS can choke on certain feeds
+		eval {parse_feed($content,$url);};
 		return;
 	}
 	else
@@ -359,21 +433,31 @@ sub parse_feed
 {
 	my ($content,$url) = @_;
 	
-	# Clear old feed contents
-	$feeds{$url}->{'contents'} = undef;
+	# Clear old feedorder
 	
 	# Parse contents with XML::RSS
 	$parser->parse($$content);
 
 	# Load parsed data into hash
-	for my $item (@{$parser->{'items'}})
+	for my $count (0 .. $#{$parser->{'items'}})
 	{
-		push(@{$feeds{$url}->{'contents'}}, {
-			title	=> $item->{'title'},
-			url	=> $item->{'link'},
-			date	=> $item->{'dc'}{'date'},
-		});
+		my $item = ${$parser->{'items'}}[$count];
+		
+		unless (exists $feeds{$url}->{'contents'}{$item->{'title'}}) {
+			$feeds{$url}->{'contents'}{$item->{'title'}} =  {
+				url	=> $item->{'link'},
+				title	=> $item->{'title'},
+			};
+		}
+		
+		unless (exists $feeds{$url}->{'contents'}{$item->{'title'}}{'read'})
+		{
+			$feeds{$url}->{'contents'}{$item->{'title'}}{'read'} = FALSE;
+		}
+		
+		@{$feeds{$url}->{'order'}}[$count] = $feeds{$url}{'contents'}{$item->{'title'}};
 	}
+	splice @{$feeds{$url}->{'order'}},scalar @{$parser->{'items'}};
 }
 
 # Update all of the feeds
@@ -389,7 +473,7 @@ sub update_feed_all
 
 	# Set to black pixmap
 	$image->set_from_pixbuf($pixbuf);
-
+	
 	return TRUE;
 }
 
@@ -413,7 +497,7 @@ sub update_feed
 #
 sub menu_show
 {
-	$menu->popup(undef, undef, undef, undef, 1, 0);
+		$menu->popup(undef, undef, undef, undef, 1, 0);
 }
 
 # Create the menu for the tray
@@ -435,13 +519,24 @@ sub menu_create
 		for my $article (0 .. ($maxfeeds-1))
 		{
 			next if (not defined 
-				$feeds{$feed}->{'contents'}->[$article]{'title'});
-				
-			my $submenuitem = Gtk2::MenuItem->new(
-				$feeds{$feed}->{'contents'}->[$article]{'title'});
+				$feeds{$feed}->{'order'}[$article]{'title'});
 				
+			my $submenuitem = Gtk2::ImageMenuItem->new(
+				$feeds{$feed}->{'order'}[$article]{'title'});
+
+			if ($feeds{$feed}->{'order'}[$article]{'read'} == FALSE)
+			{
+				my $article_image = Gtk2::Image->new_from_pixbuf($dot);
+				$submenuitem->set_image($article_image);
+			}
+			else
+			{
+				my $article_image = Gtk2::Image->new_from_pixbuf($blank);
+				$submenuitem->set_image($article_image);
+			}
+
 			$submenuitem->signal_connect('activate',\&url_load, 
-				$feeds{$feed}->{'contents'}->[$article]{'url'});
+				[$article,$feed]);
 				
 			# Attach new menu item to the sub menu
 			$submenu->append($submenuitem);
@@ -450,7 +545,9 @@ sub menu_create
 		$submenu->append($subseparator);
 		
 		# Update this feed button...
-		my $subupdate = Gtk2::MenuItem->new('Update this feed');
+		my $subupdate = Gtk2::ImageMenuItem->new('Update this feed');
+		my $refresh_image = Gtk2::Image->new_from_stock('gtk-refresh','menu');
+		$subupdate->set_image($refresh_image);
 		$subupdate->signal_connect('activate',\&update_feed,$feed);
 		$submenu->append($subupdate);
 		
@@ -463,16 +560,22 @@ sub menu_create
 		$menu->append($menuitem);
 	}
 	# Quit button...
-	my $menuquit = Gtk2::MenuItem->new('Quit');
+	my $menuquit = Gtk2::ImageMenuItem->new('Quit');
 	$menuquit->signal_connect('activate', sub { write_config(); exit 1; });
+	my $quit_image = Gtk2::Image->new_from_stock('gtk-quit','menu');
+	$menuquit->set_image($quit_image);
 	
 	# Preferences button...
-	my $menupref = Gtk2::MenuItem->new('Preferences');
+	my $menupref = Gtk2::ImageMenuItem->new('Preferences');
 	$menupref->signal_connect('activate',\&prefs_show);
+	my $prefs_image = Gtk2::Image->new_from_stock('gtk-preferences','menu');
+	$menupref->set_image($prefs_image);
 	
 	# Update all feeds button...
-	my $menuupdate = Gtk2::MenuItem->new('Update');
+	my $menuupdate = Gtk2::ImageMenuItem->new('Update');
+	my $refresh_image = Gtk2::Image->new_from_stock('gtk-refresh','menu');
 	$menuupdate->signal_connect('activate',\&update_feed_all);
+	$menuupdate->set_image($refresh_image);
 	
 	# Horz. Separator
 	my $separator = Gtk2::SeparatorMenuItem->new;
@@ -491,33 +594,39 @@ sub menu_create
 # 
 sub url_load
 {
-	# Open a URL in the specified browser
-	my ($widget, $url) = @_;
+	my ($widget, $feed_info) = @_;
+	my ($article, $feed) = @$feed_info;
+
+	$feeds{$feed}->{'order'}[$article]{'read'} = TRUE;
+
+	my $url = $feeds{$feed}->{'order'}[$article]{'url'};
 
-	# Check if we should use gnome default
 	if ($usegnome == TRUE)
 	{
 		Gnome2::URL->show($url);
 	}
 	else
 	{
-		if (-e $browser and fork == 0)
+		if (fork == 0)
 		{
-			exec($browser,$url) or 
+			my $b = $browser;
+			$b .= ' %s' unless $b =~ /\%s/;
+			exec(sprintf($b,$url)) or 
 				warn "Unable to launch browser!\n";
 		}
 	}
+	Gtk2->main_iteration while Gtk2->events_pending; 
+	menu_create();
 }
 
 # Preference window
 # 
 sub prefs_show
 {
-	update_feedlist();
-	update_simplelist();
 	my $interval_display = $gld->get_widget('interval_entry');
 	my $headings_display = $gld->get_widget('headings_entry');
 	my $browser_display = $gld->get_widget('browser_entry');
+	my $checkbutton = $gld->get_widget('use_default_browser_checkbox');
 
 	# Convert milliseconds to minutes
 	my $display_interval = ($interval / 1000) / 60;
@@ -527,6 +636,9 @@ sub prefs_show
 	$headings_display->set_text($maxfeeds);
 	$browser_display->set_text($browser);
 
+	$checkbutton->set_active($usegnome);
+	$browser_display->set_sensitive(!$usegnome);
+
 	$prefs_window->show_all;
 }
 
@@ -564,6 +676,7 @@ sub on_pref_ok_button_clicked
 	update_interval($new_interval);
 	update_maxfeeds($new_maxfeeds);
 	update_feeds_enabled();
+	write_config();
 }
 
 sub on_pref_cancel_button_clicked
@@ -658,11 +771,8 @@ sub on_add_ok_button_clicked
 	# Add entry to the info hash
 	$feeds{$address_temp->get_text}->{'title'} = $name_temp->get_text;
 	$feeds{$address_temp->get_text}->{'enable'} = NEW;
+	push @{$feedlist->{data}}, [$name_temp->get_text,NEW,$address_temp->get_text];
 	
-	# Add entry to feedlist array
-	update_feedlist();
-	update_simplelist();
-
 	# Hide and clear dialog
 	$add_dialog->hide;
 	add_dialog_clear();
@@ -703,6 +813,7 @@ sub get_feed_failed
 	{
 		# Remove
 		remove_feed($url);
+		update_feedlist();
 	}
 	if (2 == $response)
 	{
@@ -716,6 +827,8 @@ sub disable_feed
 {
 	my $url = shift;
 	$feeds{$url}->{'enable'} = FALSE;
+	update_feedlist();
+	write_config();
 }
 
 # Remove feed button
@@ -726,12 +839,13 @@ sub on_remove_button_clicked
 	my @selected = $feedlist->get_selected_indices;
 
 	# Delete rows from feed hash and feed list
-	unless ($#feedlist == -1) {
-		remove_feed($feedlist[$_][2]) for(@selected);
-		update_feedlist();
-		update_simplelist();
+	for my $selection (@selected)
+	{
+		unless ($selection == -1) {
+				remove_feed($feedlist->{data}[$selection][2]);
+				splice(@{$feedlist->{data}},$selection,1);
+		}
 	}
-	menu_create();
 }
 
 sub remove_feed
@@ -740,25 +854,21 @@ sub remove_feed
 
 	# Delete feed from hash
 	delete $feeds{$url};
-}	
 
-sub update_simplelist
-{
-	@{$feedlist->{data}} = @feedlist;
-}
+	write_config();
+}	
 
 sub update_feedlist
 {
-	# Empty feedlist
-	@feedlist = ();
-	
-	# Fill feedlist with feed info from hash
-	for my $feed (sort { lc $feeds{$a}->{'title'} cmp lc $feeds{$b}->{'title'} } keys %feeds)
-	{
-		push @feedlist, [$feeds{$feed}->{'title'}, 
-			$feeds{$feed}->{'enable'},$feed];
-	}
-}
+        # Empty feedlist
+        @{$feedlist->{data}} = ();
+        
+        # Fill feedlist with feed info from hash
+        for my $feed (keys %feeds)                                                                       { 
+                push @{$feedlist->{data}}, [$feeds{$feed}->{'title'},
+                        $feeds{$feed}->{'enable'},$feed];
+        }               
+}    
 
 sub get_favicon
 {
@@ -808,18 +918,20 @@ sub on_use_default_browser_checkbox_toggled
 	
 	my $browser_entry = $gld->get_widget('browser_entry');
 
-	# Check if checkbutton is clicked
+	# Check if checkbutton is active
 	if ($widget->get_active)
 	{
-		# If so make browser command uneditable
-		$browser_entry->set_editable(FALSE);
-		$browser_entry->set_visibility(FALSE);
+		$browser_entry->set_sensitive(FALSE);
 	}
 	else
 	{
-		# If not make browser command editable
-		$browser_entry->set_editable(TRUE);
-		$browser_entry->set_visibility(TRUE);
+		$browser_entry->set_sensitive(TRUE);
 	}
 }
 
+sub on_about_button_clicked
+{
+	my $about = Gnome2::About->new(
+		$name,$version,$copyright,$comments,$authors);
+	$about->show;
+}
